--[[
************************************************************************
Project				: SKG
Author				: zhinjio
Project Revision	: r140
Project Date		: 20090515231408

File				: SKG_UtilityFunctions.lua
Commit Author		: zhinjio
Commit Revision		: 140
Commit Date			: 20090515231408
************************************************************************
Description	:
	This isn't the best way to do this, since these all get global
	declares. Maybe I'll work that out sometime. In any case, these are
	functions that are used by different areas of the code. In some case,
	they're only used once, and should be pulled in to the appropriate
	other file. Someday I'll do that. Maybe.

TODO		:
	Check to see if SKG_PrettyTimeStamp can be removed?
	The following are candidates for moving to SKG.lua:
		SKG_RepairLists(), SKG_checkList(), SKG_ListError()

************************************************************************
(see bottom of file for changelog)
************************************************************************
--]]

local SKG = LibStub( "AceAddon-3.0"):GetAddon( "SKG" )
local SKGL = LibStub( "AceLocale-3.0"):GetLocale( "SKG" )

-- Assign variables that require localizations to be loaded already

SKG_ClassNames = { 
	SKGL["SKG_CLASS_DRUID"],			-- 1
	SKGL["SKG_CLASS_HUNTER"],			-- 2
	SKGL["SKG_CLASS_MAGE"],				-- 3
	SKGL["SKG_CLASS_PALADIN"],			-- 4
	SKGL["SKG_CLASS_PRIEST"],			-- 5
	SKGL["SKG_CLASS_ROGUE"],			-- 6
	SKGL["SKG_CLASS_SHAMAN"],			-- 7
	SKGL["SKG_CLASS_WARLOCK"],			-- 8
	SKGL["SKG_CLASS_WARRIOR"],			-- 9
	SKGL["SKG_CLASS_DEATHKNIGHT"],		-- 10
}

SKG_enClassNames = {
	"DRUID",
	"HUNTER",
	"MAGE",
	"PALADIN",
	"PRIEST",
	"ROGUE",
	"SHAMAN",
	"WARLOCK",
	"WARRIOR",
	"DEATHKNIGHT",
}

-- Seed the random number generator:
-- this function was removed for 3.1. Wow, how lame is that.
-- math.randomseed(math.random(0,2147483647)+(GetTime()*1000));

function SKG_getVersionComponents()
	local major, minor = 0, 0
	-- SKG_VERSION will have two forms. If we're testing, and not at a tagged push,
	-- it will be:
	--		rXXXX
	-- for example (r121)
	-- on the other hand, if this is a tagged release:
	--		Major.Minor.Bugfix-Tag
	-- for example (3.0.2-release)
	if ( string.sub( SKG_VERSION, 1, 1 ) == 'r' ) then
		major = string.sub( SKG_VERSION, 2 )
	else
		major, minor = string.match( SKG_VERSION, "(%d+)%.(%d+)%.%d+%-%a+" )
	end
	--[[ old code
	local tmp = SKG_VERSION * 100
	local major = math.floor( tmp / 100 )
	local minor = tmp - ( major * 100 )
	--]]
	return major, minor
end

function SKG_Print( msg )
	local _, fh = DEFAULT_CHAT_FRAME:GetFont()
	fh = math.ceil( fh )
	local icon = "\124TInterface\\Addons\\SKG\\BLP\\ChatIcon:" .. fh .. "\124t"
	if not ( msg == nil ) then
	   if( DEFAULT_CHAT_FRAME ) then
	      DEFAULT_CHAT_FRAME:AddMessage( icon .. " SKG: " .. msg, 0.6, 1.0, 1.0 )
	   end
	end
end

function SKG_Debug( msg )
	if not ( msg == nil ) then
		if ( SKG_DEBUG ) then
			SKG_Print( "DEBUG: " .. msg )
		end
	end
end

function SKG_SetSecurityLevel()
	-- I might not be in a guild... weird, huh?
	if ( IsInGuild() == 1 ) then
		SKG_Debug( "I'm in a guild" )
		-- Figure out my rank to send
		local myRank = 20
		for i = 1, GetNumGuildMembers( true ) do
			local gName, _, gRankIndex = GetGuildRosterInfo(i);
			if ( gName == SKG.me ) then
				myRank = gRankIndex
				break
			end
		end
		SKG_Debug( "My guild rank index is " .. myRank )
		-- Read the guild note, and look for our security info
		local rankAuthValue = 0
		local manualNames = ""
		local guildNote = GetGuildInfoText()
		if ( guildNote ~= nil ) then
			SKG_Debug( "I found a valid guild note" )
			rankAuthValue, manualNames = string.match( guildNote, ".*%[SKG/R:(%d+)/(%a*)//%].*" )
			if ( rankAuthValue ~= nil ) then
				SKG_Debug( "rankauthvalue is " .. rankAuthValue )
				if ( myRank <= tonumber( rankAuthValue ) ) then
					SKG_Debug( "I match the current rank check, setting to S" )
					SKG_SECURITY_LEVEL = "S"
				end
			end
			if ( manualNames ~= nil ) then
			SKG_Debug( "manualnames is " .. manualNames )
				manualNames = "," .. manualNames .. ","
				local pattern = "," .. SKG.me .. ","
				if ( string.find( manualNames, pattern ) ~= nil ) then
					-- name has been found
					SKG_Debug( "I match the current manual auth list, setting to S" )
					SKG_SECURITY_LEVEL = "S"
				end
			end
		end
		-- If we're on a guild leader, set his security level anyway
		if ( IsGuildLeader( SKG.me ) ) then 
			SKG_SECURITY_LEVEL = "S"
			SKG_Debug( "you are the guild master" )
		end
	else
		SKG_Debug( "I'm not in a guild" )
		-- If I'm not in a guild, I should have this enabled. Its essentially useless
		SKG:Disable()
	end
end

function SKG_TooltipDisplay( this, textLabel, useTrans )
	this:SetScript( "OnEnter",
		function ( this )
			GameTooltip_SetDefaultAnchor( GameTooltip, this )
			if ( useTrans == 1 ) then
				-- I'm gonna try and figure out which translation table to use here
				-- since I'd rather not add code elsewhere
				GameTooltip:SetText( SKGL[ textLabel ], HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			else
				GameTooltip:SetText( textLabel, HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b )
			end
			GameTooltip:Show()
		end
	)
	this:SetScript( "OnLeave",
		function( this )
			GameTooltip:Hide()
		end
	)
end

function SKG_DisableNormalText( textObj )
	textObj:SetTextColor( .4, .4, .4, 1 )
end

function SKG_EnableNormalText( textObj )
	textObj:SetTextColor( 1, .82, 0, 1 )
end

function IsOdd( value )
	if ( floor( value / 2 ) == floor( ( value + 1 ) / 2 ) ) then
		return false
	else
		return true
	end
end

function SKG_UpdateRaiders()
	SKG.Raiders = {}
	local numRaiders = GetNumRaidMembers()
	local plList = SKG.db.profile.Players
	local altList = SKG.db.profile.Alts
	local mainPID = 0
	local isAlt = false
	for i = 1, 40 do
		local currUnit = "raid" .. i
		if ( UnitExists( currUnit ) ) then
			mainPID = 0
			isAlt = false
			local raidName = GetRaidRosterInfo(i);
			local _, raidClass = UnitClass( currUnit )

			-- Its possible for UnitClass to return nil for raidClass if the unit
			-- in question is DC'd or otherwise not viewable. Fail silently and
			-- continue looping if that is the case
			if ( raidClass ~= nil ) then
				-- we want to store this person's PID, if he's in our raid list
				local found = false
				raidClass = strupper( raidClass )
				for	PID, plInfo in pairs( plList ) do
					if ( plInfo.name == raidName ) then
						mainPID = PID
						isAlt = false
						found = true
						break
					end
				end
				-- if we didn't find him in the main list, is it possibly an Alt of someone?
				if ( found == false ) then
					for PID, altInfo in pairs( altList ) do
						for j = 1, #altInfo do
							if ( altInfo[j].name == raidName ) then
								mainPID = PID
								isAlt = true
								found = true
								break
							end
						end
						if ( found == true ) then
							break
						end
					end
				end
				local t = {
					name = raidName,
					class = raidClass,
					raidID = i,
					mPID = mainPID,
					iAlt = isAlt
				}
				tinsert( SKG.Raiders, t )
			end
		end
	end
end

local function SKG_TimeStamp()
	-- return datestamp formatted as YYYYMMDDHHMMSS, use UTC time
	local blort = date("!*t")

	local outString = blort.year
	if ( blort.month < 10 ) then
		outString = outString .. "0" .. blort.month
	else
		outString = outString .. blort.month
	end
	if ( blort.day < 10 ) then
		outString = outString .. "0" .. blort.day
	else
		outString = outString .. blort.day
	end
	if ( blort.hour < 10 ) then
		outString = outString .. "0" .. blort.hour
	else
		outString = outString .. blort.hour
	end
	if ( blort.min < 10 ) then
		outString = outString .. "0" .. blort.min
	else
		outString = outString .. blort.min
	end
	if ( blort.sec < 10 ) then
		outString = outString .. "0" .. blort.sec
	else
		outString = outString .. blort.sec
	end

	return outString
end

local function SKG_DaysSinceEpoch( inString )
--	SKG_Debug( "SKG_DaysSinceEpoch: " .. inString )
	inYear = string.sub( inString, 7, 10 )
	inYear = inYear - 1
	inMonth = string.sub( inString, 1, 2 )
	inMonth = inMonth - 1
	inDay = string.sub( inString, 4, 5 )
	inDay = inDay - 1 + 1
	return ( inYear * 365 ) + ( inMonth * 30 ) + ( inDay * 1 )
end

function SKG_TimeSince( inString )
	-- takes a string in format YYYYMMDDHHMMSS
	-- and returns the time in days since last activity (eg, "3 days")
	local currentDate = date( "!*t" )
--	SKG_Debug( "SKG_TimeSince: " .. inString )
	local inYear = string.sub( inString, 1, 4 )
	local inMonth = string.sub( inString, 5, 6 )
	local inDay = string.sub( inString, 7, 8 )

	local outString = ""
	if ( currentDate.month < 10 ) then
		outString = "0" .. currentDate.month .. "/"
	else
		outString = currentDate.month .. "/"
	end
	if ( currentDate.day < 10 ) then
		outString = outString .. "0" .. currentDate.day .. "/"
	else
		outString = outString .. currentDate.day .. "/"
	end
	outString = outString .. currentDate.year

	return SKG_DaysSinceEpoch( outString ) - SKG_DaysSinceEpoch( inMonth .. "/" .. inDay .. "/" .. inYear ) .. " " .. SKGL["SKG__DAYS"]
end

function SKG_CreateButton(
	bName, parentFrame,	bHeight, bWidth,
	anchorFrom, anchorFrame, anchorTo, xOffset, yOffset,
	bNormFont, bHighFont, initText, tAlign, tooltipText, noTextures )

	-- I hate stretchy buttons. Thanks very much to ckknight for this code
	-- (found in RockConfig)

	-- when pressed, the button should look pressed
	local function button_OnMouseDown(this)
		if this:IsEnabled() == 1 then
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Down]])
		end
	end
	-- when depressed, return to normal
	local function button_OnMouseUp(this)
		if this:IsEnabled() == 1 then
			this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
			this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		end
	end

	local function button_Disable(this)
		this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Disabled]])
		this:__Disable()
		this:EnableMouse(false)
	end

	local function button_Enable(this)
		this.left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this.middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this.right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		this:__Enable()
		this:EnableMouse(true)
	end

	local button = CreateFrame( "Button", bName, parentFrame )

	button:SetWidth( bWidth )
	button:SetHeight( bHeight )

	if not ( noTextures == 1 ) then
		local left = button:CreateTexture(button:GetName() .. "_LeftTexture", "BACKGROUND")
		button.left = left
		local middle = button:CreateTexture(button:GetName() .. "_MiddleTexture", "BACKGROUND")
		button.middle = middle
		local right = button:CreateTexture(button:GetName() .. "_RightTexture", "BACKGROUND")
		button.right = right

		left:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		middle:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])
		right:SetTexture([[Interface\Buttons\UI-Panel-Button-Up]])

		left:SetPoint("TOPLEFT")
		left:SetPoint("BOTTOMLEFT")
		left:SetWidth(12)
		left:SetTexCoord(0, 0.09375, 0, 0.6875)

		right:SetPoint("TOPRIGHT")
		right:SetPoint("BOTTOMRIGHT")
		right:SetWidth(12)
		right:SetTexCoord(0.53125, 0.625, 0, 0.6875)

		middle:SetPoint("TOPLEFT", left, "TOPRIGHT")
		middle:SetPoint("BOTTOMRIGHT", right, "BOTTOMLEFT")
		middle:SetTexCoord(0.09375, 0.53125, 0, 0.6875)

		button:SetScript("OnMouseDown", button_OnMouseDown)
		button:SetScript("OnMouseUp", button_OnMouseUp)
		button:SetScript("OnEnter", SubControl_OnEnter)
		button:SetScript("OnLeave", SubControl_OnLeave)

		button.__Enable = button.Enable
		button.__Disable = button.Disable
		button.Enable = button_Enable
		button.Disable = button_Disable

		local highlight = button:CreateTexture(button:GetName() .. "_Highlight", "OVERLAY", "UIPanelButtonHighlightTexture")
		button:SetHighlightTexture(highlight)
	end

	local text = button:CreateFontString(button:GetName() .. "_FontString", "ARTWORK")
	button:SetFontString(text)
	button.text = text
	text:SetPoint("LEFT", button, "LEFT", 7, 0)
	text:SetPoint("RIGHT", button, "RIGHT", -7, 0)
	text:SetJustifyH( tAlign )

	button:SetNormalFontObject(bNormFont)
	button:SetHighlightFontObject(bHighFont)
	button:SetDisabledFontObject(GameFontDisableSmall)

	text:SetText( initText )		

	button:SetPoint( anchorFrom, anchorFrame, anchorTo, xOffset, yOffset )
	if ( tooltipText ~= "" ) then
		SKG_TooltipDisplay( button, tooltipText, 1 )
	end

	return button
end

function SKG_AddLogEntry( TID, List, CommandToken, Payload, newlistHash )
	if ( type( Payload ) == "table" ) then
		SKG_Debug( "ALE : TID " .. TID .. ", List " .. List ..
			", Token " .. CommandToken .. ", PL is table" )
	else
		SKG_Debug( "ALE : TID " .. TID .. ", List " .. List ..
			", Token " .. CommandToken .. ", PL " .. Payload )
	end

	local db = SKG.db.profile.Log
	-- If we're not a syncer, don't log
	if ( SKG_SECURITY_LEVEL ~= "U" ) then
		local addEntry = 0
		if ( List == "INIT" ) then
			-- This is our first entry. Just add it
			List = "SKG"
			db[List] = {}
			addEntry = 1
		else
			-- Does a log entry exist already for the current TID?
			if ( not db[List][TID] ) then
				addEntry = 1
			else
				-- We've already got one, you see. Go away or I shall taunt
				-- you a second time
				SKG_Debug( "log entry already existed" )
			end
		end
		if ( addEntry == 1 ) then
			SKG_Debug( "addentry was 1, attempting to add" )
			local t = {
				token = CommandToken,
				payload = Payload,
				nlHash = newlistHash
			}
			db[List][TID] = {}
			db[List][TID] = t
		end
	end
end

function SKG_CreateTID( index, itemid, uniqueid )
	local outString = SKG_TimeStamp()
	outString = outString .. index
	-- pad the itemid to make it 8 characters
	outString = outString .. strrep( "0", ( 8 - strlen( itemid ) ) ) .. itemid
	-- documentation shows that uniqueid can be negative. make it positive
	outString = outString .. abs( uniqueid )
	return outString
end

local function SKG_unhashName( name )
	local outString = ""
	for nv in string.gmatch( name, "(..)" ) do
		outString = outString .. string.char( "0x" .. nv )
	end
	return outString
end

function SKG_HashName( name )
	-- Since usernames are, at maximum 12 unicode characters, my output should 2 x 12 = 24 hex characters
	-- Note, for unicode special characters (eg, \195\167), the \195 part will be ignored
	local outString = ""
	for i = 1, strlen( name ) do
		local bv = strbyte( name, i )
		if ( bv == 195 ) then
			-- ignore. I don't want the special codes. just the characters
		else
			outString = outString .. string.format( "%x", bv )
		end
	end
	while ( strlen( outString ) < 24 ) do
		outString = outString .. "32"
	end	
	return strupper( outString )
end

function SKG_HashRollList( TID, rollList )
	-- Method:
	--	Step 1: 
	--		convert each letter in a name to a number (1-255)
	--		add them together (max value : 12 * 255 = 3060
	--		add the list position 3060 + 450 = 3510
	--	Step 2:
	--		add groups of 9 names together ( 9 * 3510 = 31590)
	--		convert to hex (4 characters)
	--
	-- Non-arbitrary limitation of lists to 450 raiders
	-- This will yield 450/9 = 50 groups x 4 characters = 200 character hash
	-- Timestamp + the above string is the FULL hash of the list, yielding a maximum
	-- of 214 characters
	-- 
	-- For display purposes (sync page), we need to rehash to a smaller string
	--
	--	Step 3:
	--		take groups of 16 hex characters, and add them together
	--		highest possible value = 16 x 16 = 256
	--		divide by 2, highest value = 128 (2 hex characters)
	--		177 / 16 = 12 sets * 2 characters = 24 characters
	--	Step 4:
	--		add in the Timestamp digits, don't carry ones
	--
	-- Thus, the shorthash will ALWAYS be 24 "digits"
	--
	-- If the list in question is "Players" then, this simplifies considerably
	-- since I don't care about the TID or list position. Just add everything
	-- together. The format for returned data is:
	--		longhash = PlayerHash (#Players), shorthash = AltHash (#Alts)
	--

	local outString = ""
	local tempValue = 0
	local altValue = 0
	local newHash = ""
	if ( rollList == "Players" ) then
		local numPlayers = 0
		for nameHash, playerTable in pairs( SKG.db.profile.Players ) do
			numPlayers = numPlayers + 1
			local tName = playerTable.name
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
		end
		-- convert to hex
		outString = outString .. string.format( "%x", tempValue )
		if ( string.len( outString ) > 8 ) then
			outString = string.sub( outString, 1, 8 )
		end
		outString = outString .. "(" .. numPlayers .. ")"

		local altPlayers = 0
		dbAlts = SKG.db.profile.Alts
		for hash, altList in pairs( dbAlts ) do
			for altIndex, plInfo in pairs( dbAlts[hash] ) do
				altPlayers = altPlayers + 1
				local tName = altList[altIndex].name
				for k = 1, strlen( tName ) do
					altValue = altValue + strbyte( tName, k )
				end
			end
		end
		newHash = newHash .. string.format( "%x", altValue )
		if ( string.len( newHash ) > 8 ) then
			newHash = string.sub( newHash, 1, 8 )
		end
		newHash = newHash .. "(" .. altPlayers .. ")"
	else
		outString, newHash = SKG_GenericListHash( TID, SKG.db.profile.RollLists[rollList] )
	end

	return strupper( outString ), strupper( newHash )
end

function SKG_InitAllPanels()
	SKG_Debug( "In InitAllPanels" )
	-- We've switched profiles, and have new settings. Make sure all options
	-- and checkboxes are appropriately set
	local db = SKG.db.profile

	-- If we've switched to an empty profile, at least start with the defaults
	-- This code shouldn't be necessary. We should automatically get defaults...

	-- Start with Options panel
	local dbGO = db.Options.General
	dbGO.xPos = SKG_Frame:GetLeft()
	dbGO.yPos = SKG_Frame:GetTop()
	SKG_OF_RaidOnlyCB:SetChecked( dbGO.raidonly )
	SKG_OF_ShowAllCB:SetChecked( not dbGO.raidonly )
	SKG_OF_ShowClassColorsCB:SetChecked( dbGO.classColors )
	SKG_SetOptionSwitcherButtonTexture( dbGO.buttonTexture )
	local profile = SKG.db:GetCurrentProfile()
	if ( profile == "Default" ) then
		SKG_OF_ProfileCB:SetChecked( true )
		SKG_OF_ProfileCopy:Disable()
	else
		SKG_OF_ProfileCB:SetChecked( false )
		SKG_OF_ProfileCopy:Enable()
	end
	SKG_OF_AutoOpenCB:SetChecked( dbGO.autoopen )
	SKG_DEBUG = dbGO.debug
	SKG_OF_DebugCB:SetChecked( dbGO.debug )

	-- Lists:
	-- Initialize the dropdown list with the current saved lists...
	SKG_UpdateLists()

	local dbLO = SKG.db.profile.Options.List
	if not ( dbLO.AutoAdd ) then
		dbLO.AutoAdd = false
		SKG_LF_AutoAddCB:SetChecked( false )
	else
		dbLO.AutoAdd = true
		SKG_LF_AutoAddCB:SetChecked( true )
	end

	-- verify that we have a "Loot" list, since this
	-- was added in a later release
	if not ( db.Log["Loot"] ) then
		db.Log["Loot"] = {}
	end

	-- Bid stuff:
	-- Unfortunately, there are some versions of code out there that never
	-- set this value's default properly. If it doesn't exist, set it.
	local dbBO = SKG.db.profile.Options.Bid
	if not ( dbBO.ArmorFilter ) then
		dbBO.ArmorFilter = 0
	end
	if ( dbBO.ArmorFilter == 0 ) then
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_NONEFULL"] )
	elseif ( dbBO.ArmorFilter == 1 ) then
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_LOOSEFULL"] )
	else
		SKG_BBF_ArmorFilterText:SetText( SKGL["SKG_LBL_BBF_ARMORFILTER_TEXT_STRICTFULL"] )
	end

	SKG_UpdateLootList()

	-- check if we're in a raid.
	local numRaiders = GetNumRaidMembers()
	if ( numRaiders > 0 ) then
		SKG.inRaid = true
		SKG_UpdateRaiders()

		SKG_LF_SpamListButton:Enable()
		-- if we also happen to be raid leader, then enable this stuff too
		if ( IsRaidLeader() == 1 ) then
			SKG_BF_LootTypeButton:Show()
			SKG_BF_LootThreshButton:Show()
			SKG_BF_LootTypeButton:Enable()
			SKG_BF_LootThreshButton:Enable()

			-- Build the initial list of people in the ML list.
			-- scan through the SKG.Raiders struct, and put entries in the list
			SKG.lootTypeSelections.args[3].args = {}
			for i = 1, #SKG.Raiders do
				raidName = SKG.Raiders[i].name
				raidIndex = SKG.Raiders[i].raidID
				SKG_Debug( "n = " .. raidName .. ", rI = " .. raidIndex )
				local a = {
					type = "execute",
					name = raidName,
					desc = raidName,
					func = function( value )
						-- Set Master Looter to this player
						SetLootMethod( "master", value )
						Dewdrop:Close()
					end,
					passValue = raidName,
					order = raidIndex,
				}
				table.insert( SKG.lootTypeSelections.args[3].args, a )
			end
		else
			SKG_BF_LootTypeButton:Hide()
			SKG_BF_LootThreshButton:Hide()
		end
	else
		SKG_BF_LootTypeButton:Hide()
		SKG_BF_LootThreshButton:Hide()
		SKG_LF_SpamListButton:Disable()
	end
	-- Disabled until looting events occur
	SKG_UpdateBidderPanel()
	SKG_BF_BidButton:Disable()
	SKG_BF_RetractButton:Disable()
	SKG_BidButtonFrame:Hide()

	-- Text for the title...
	local major, minor = SKG_getVersionComponents()
	SKG_TitleText:SetText( string.format( SKGL["SKG_LBL_TITLE_TEXT"] , 
						major, minor, SKG_SECURITY_LEVEL) )

	-- Display lists now...
	SKG_UpdateLists()
	-- Update (create) the fubar tooltip
--	SKG:UpdateFuBarTooltip()
end

function SKG_SetLootFilter( bitMap )
	SKG.LootFilter = bitMap
	for index = 1, SKG.numClasses do
		local tempT = SKG_BFF_ClassFilterText[index]
		if ( string.sub( bitMap, index, index ) == "1" ) then
			tempT:SetText( "[ X ] " .. SKG_ClassNames[index] )
		else
			tempT:SetText( "[ _ ] " .. SKG_ClassNames[index] )
		end
	end
	SKG_Debug( "LF = " .. SKG.LootFilter )
end

function SKG_SetSingleLootFilter( bitMap )
	local index = strfind( bitMap, "1" )

	local beginS, endS, currV = ""
	beginS = strsub( SKG.LootFilter, 0, (index - 1) )
	endS = strsub( SKG.LootFilter, (index + 1) )
	currV = strsub( SKG.LootFilter, index, index )
	local tempT = SKG_BFF_ClassFilterText[index]
	if ( currV == "1" ) then
		SKG.LootFilter = beginS .. "0" .. endS
		tempT:SetText( "[ _ ] " .. SKG_ClassNames[index] )
	else
		SKG.LootFilter = beginS .. "1" .. endS
		tempT:SetText( "[ X ] " .. SKG_ClassNames[index] )
	end
	SKG_Debug( "LF = " .. SKG.LootFilter )
end

function SKG_CombineLootFilters( )
	SKG_Debug( "LF = " .. SKG.LootFilter )
	SKG_UpdateLootFilters()
	local retFilter = ""
	for i = 1, SKG.numClasses do
		local mf = strsub( SKG.LootFilter, i, i )
		local af = strsub( SKG.afModify, i, i )
		if ( ( mf == "1" ) and ( af == "1" ) ) then
			retFilter = retFilter .. "1"
		else
			retFilter = retFilter .. "0"
		end
	end
	return retFilter
end

function SKG_UpdateLootFilters( ) 
	-- if we have an item selected, go ahead and get details, update
	-- SKG.afModify based on that.
	if ( SKG.selectedBidItem ~= 0 ) then
		local itemLink = SKG_BF_BidItems[SKG.selectedBidItem]:GetText()

		local itemID = string.match( itemLink, "item:(%d+)" ) + 0
		local _, _, _, _, _, itemType, itemSubType = GetItemInfo( itemLink )

		if ( SKG_LootFilter[itemID] ) then
			local thisLF = SKG_LootFilter[itemID]
			SKG_SetLootFilter( thisLF )
			SKG_Print( SKGL["SKG_MSG_LFSET"] )
		end

		-- Change our armor modifier, if appopriate
		if ( itemType == "Armor") then
			if ( ( itemSubType == "Cloth" ) or
				 ( itemSubType == "Leather" ) or
				 ( itemSubType == "Mail" ) or
				 ( itemSubType == "Plate" ) ) then
				local af = SKG.db.profile.Options.Bid.ArmorFilter
				SKG.afModify = SKG_ArmorFilter[af][itemSubType]
			else
				SKG.afModify = "1111111111"
			end
		else
			SKG.afModify = "1111111111"
		end
	end
end

function SKG_ClassifyLootFilter( filter )
	local retStr = ""
	for i = 1, SKG.numClasses do
		local f = strsub( filter, i, i )
		if ( f == "1" ) then
			if ( retStr == "" ) then
				retStr = retStr .. SKG_ClassNames[i]
			else
				retStr = retStr .. " " .. SKG_ClassNames[i]
			end
			SKG_Debug( retStr )
		end
	end

	return retStr
end

function SKG_GetMyRaidID( )
	local unitName = ""
	for index = 1, 40 do
		unitName = GetRaidRosterInfo( index )
		if not ( unitName == nil ) then
			if ( unitName == SKG.me ) then
				return index
			end
		end
	end
	return 0
end

function SKG_GetOldestTID( logList )
	local minTID = 0
	local minTIDs = ""

	if ( logList == nil ) then
		return 0
	end
	
	for index, tval in pairs( logList ) do
		local conv = tonumber(index)
		if ( minTID == 0 ) then
			minTID = conv
			minTIDs = index
		else
			if ( conv < minTID ) then
				minTID = conv
				minTIDs = index
			end
		end
	end

	return minTIDs
end

function SKG_GetLatestTID( logList )
	local maxTID = 0
	local maxTIDs = ""

	if ( logList == nil ) then
		return 0
	end

	for index, tval in pairs( logList ) do
		local conv = tonumber( index )
		if ( conv > maxTID ) then
			maxTIDs = index
			maxTID = conv
		end
	end

	return maxTIDs
end

function SKG_AddAlphaGroups( table )
	for alphaIndex = 1, 5 do
		if ( alphaIndex == 1 ) then
			nameDesc = "A-C"
		elseif ( alphaIndex == 2 ) then
			nameDesc = "D-F"
		elseif ( alphaIndex == 3 ) then
			nameDesc = "G-L"
		elseif ( alphaIndex == 4 ) then
			nameDesc = "M-R"
		else
			nameDesc = "S-Z"
		end
		local v = {
			type = "group",
			name = nameDesc,
			desc = nameDesc,
			order = 100 + alphaIndex,
			args = { },
		}
		tinsert( table.args, v )
	end
end

function SKG_UpdateMainsChooser( )
	SKG.mainSelections.args = {}
	SKG_AddAlphaGroups( SKG.mainSelections )
	SKG.mainPID = ""
	local dblist = SKG.db.profile.Players
	SKG_LFIPF_AltCB:SetChecked( false )

	SKG_Debug( "in UpdateMainsChooser" )
	if ( next( dblist ) == nil ) then
		-- we don't have any mains yet. disable the "Alt" selector
		SKG_LFIPF_MainChooser:Disable()
		SKG_LFIPF_AltCB:Disable()
		SKG_DisableNormalText( SKG_LFIPF_AltCBText )
	else 
		-- We've got entries, fill 'em in.
		SKG_LFIPF_MainChooser:Enable()
		SKG_LFIPF_AltCB:Enable()
		SKG_EnableNormalText( SKG_LFIPF_AltCBText )
		for PID,playerInfo in pairs( dblist ) do
			local pValue = PID .. "|" .. dblist[PID].name
			local t = {
				type = "execute",
				name = dblist[PID].name,
				desc = dblist[PID].name,
				func = function ( crapola )
					pID, pName = strsplit( "|", crapola )
					SKG.mainPID = pID
					SKG_LFIPF_MainChooser:SetText( SKGL["SKG_LBL_LFIPF_SELECTMAIN_TEXT"] .. ": " .. pName )
					Dewdrop:Close()
				end,
				passValue = pValue
			}
			local AI = alphaIndex( dblist[PID].name )
			tinsert( SKG.mainSelections.args[AI].args, t )
		end
	end
end

function SKG_UpdateToonChooser( )
	SKG.toonSelections.args = {}
	SKG_AddAlphaGroups( SKG.toonSelections )

	SKG.selectedToon = ""
	SKG.mainPID = ""
	local dblist = SKG.db.profile.Players
	local altlist = SKG.db.profile.Alts

	SKG_Debug( "in UpdateToonChooser" )
	if ( next( dblist ) == nil ) then
		-- we don't have any mains yet. disable the "Alt" selector
		SKG_LFTCF_ToonChooser:Disable()
	else 
		-- We've got entries, fill 'em in.
		SKG_LFTCF_ToonChooser:Enable()
--		for PID,playerInfo in pairs( dblist ) do
		for index = 1, #SKG.SPlayers do
			PID = SKG.SPlayers[index]
--			playerInfo = dblist[PID]
			local mOrder = index * 100
			local pValue = PID .. "|" .. dblist[PID].name
			local t = {
				type = "execute",
				name = dblist[PID].name,
				desc = dblist[PID].name,
				func = function ( crapola )
					pID, pName = strsplit( "|", crapola )
					SKG.selectedToon = pName
					SKG.mainPID = pID
					SKG.isAlt = false
					SKG_LFTCF_ToonChooser:SetText( SKGL["SKG_LBL_LFTCF_SELECTTOON_TEXT"] .. ": " .. pName )
					Dewdrop:Close()
				end,
				order = mOrder,
				passValue = pValue
			}
			local AI = alphaIndex( dblist[PID].name )
			tinsert( SKG.toonSelections.args[AI].args, t )

			-- If this toon has any alts, list them here...
			if ( altlist[PID] ) then
				for i2 = 1, #altlist[PID] do
					local aOrder = mOrder + i2
					local pValue = PID .. "|" .. altlist[PID][i2].name
					local alt = {
						type = "execute",
						name = "- " .. altlist[PID][i2].name,
						desc = "Alt of " .. dblist[PID].name,
						func = function ( crapola )
							pID, pName = strsplit( "|", crapola )
							SKG.selectedToon = pName
							SKG.mainPID = pID
							SKG.isAlt = true
							SKG_LFTCF_ToonChooser:SetText( SKGL["SKG_LBL_LFTCF_SELECTTOON_TEXT"] .. ": " .. pName )
							Dewdrop:Close()
						end,
						order = aOrder,
						passValue = pValue
					}
					tinsert( SKG.toonSelections.args[AI].args, alt )
				end
			end
			index = index + 1
		end
	end
end

function alphaIndex( blort )
	local AI = 0
	firstLetter = string.sub( blort, 1, 1 )
	if ( firstLetter == "A" or
		firstLetter == "B" or
		firstLetter == "C" ) then AI = 1
	elseif ( firstLetter == "D" or
		firstLetter == "E" or
		firstLetter == "F" ) then AI = 2
	elseif ( firstLetter == "G" or 
		firstLetter == "H" or
		firstLetter == "I" or
		firstLetter == "J" or
		firstLetter == "K" or
		firstLetter == "L" ) then AI = 3
	elseif ( firstLetter == "M" or
		firstLetter == "N" or
		firstLetter == "O" or
		firstLetter == "P" or
		firstLetter == "Q" or
		firstLetter == "R" ) then AI = 4
	else AI = 5 end
	return AI
end

function SKG_UpdateSPlayers ( )
	SKG.SPlayers = {}
	for PID, pInfo in pairs( SKG.db.profile.Players ) do
		tinsert( SKG.SPlayers, PID )
	end
	table.sort( SKG.SPlayers, function ( x, y )
		if ( SKG.db.profile.Players[x].name <= SKG.db.profile.Players[y].name ) then
			return true
		else
			return false
		end
	end )
end

function SKG_GenericListHash( TID, gList )
	local tempValue = 0
	local outString = ""
	local newHash = ""

	if ( next( gList ) == nil ) then
		return 0, 0
	end

	local pList = SKG.db.profile.Players
	local timeStamp = strsub( TID, 1, 14 )
	local i, j, k = 0, 0, 0
	j = 1
	for i = 1, #gList do
		-- Add a check in here in the case where we have an entry in a roll list
		-- that shouldn't be here. It means our Players list is out of data. Let
		-- the user know
		if not ( pList[gList[i]] ) then
			if ( SKG_SECURITY_LEVEL ~= U ) then
				SKG_Print( SKGL["SKG_ERROR_MISSINGPLAYERS"] )
			else
				SKG_Print( SKGL["SKG_ERROR_MISSINGPLAYERS2"] )
			end
			return "0", "0"
		end

		local tName = pList[gList[i]].name
		if ( j == 9 ) then
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
			tempValue = tempValue + i -- adds in list position
			outString = outString .. string.format( "%x", tempValue )
			tempValue = 0
			j = 1
		else
			for k = 1, strlen( tName ) do
				tempValue = tempValue + strbyte( tName, k )
			end
			j = j + 1
		end
	end
	if not ( tempValue == 0 ) then
		outString = outString .. string.format( "%x", tempValue )
	end
	outString = timeStamp .. outString

	-- now for the shorter hash:

	tempValue = 0
	local shortHash = ""
	local moreZeroes = 16 - math.fmod( strlen( outString ), 16 )
	local newString = ""
	if ( moreZeroes < 16 ) then
		newString = strrep( "0", moreZeroes ) .. outString
	else
		newString = outString
	end
	for i = 1, ( strlen( newString ) / 16 ) do
		local portion = strsub( newString, ( 16 * ( i - 1 )) + 1, ( 16 * i ) )
		for j = 1, 16 do
			tempValue = tempValue + ( ( "0x" .. strsub( portion, j, j ) ) + 0 )
		end
		tempValue = floor( tempValue / 2 )
		shortHash = shortHash .. string.format( "%x", tempValue )
		tempValue = 0
	end
	
	-- add in the timestamp
	for index = -24, -1, 1 do
		local tsC = strsub( TID, index, index )
		local shC = strsub( shortHash, index, index )
		if ( shC == "" ) then
			newHash = newHash .. tsC
		else
			local blort = string.sub( string.format( "%X", ( "0x" .. tsC ) + ( "0x" .. shC ) ), -1, -1 )
			newHash = newHash .. blort
		end
	end
	return outString, newHash
end

function SKG_RunTransactions( transList, targetList, lastmodList )
	-- This function is just a coordinated place to put all the various
	-- handlers for each type of transaction against a roll list.
	-- "transList" is the list of transactions to run, in no particular
	-- order, which "targetList" is the Roll list to run the transactions
	-- against

	-- first, we need to sort transList to be in time order...
	SKG_Debug( "Sorting transaction list..." )
	local sortedTrans = {}
	for k, v in pairs( transList ) do
		if ( #sortedTrans == 0 ) then
			tinsert( sortedTrans, k )
		else
			local i = 1
			local found = false
			while ( ( i <= #sortedTrans ) and ( found == false ) ) do
				if ( k < sortedTrans[i] ) then
					found = true
				else
					i = i + 1
				end
			end
			if ( found == false ) then
				tinsert( sortedTrans, k )
			else
				tinsert( sortedTrans, i, k )
			end
		end
	end
	SKG_Debug( "Sorted entries " .. #sortedTrans )

	-- One of two things must be true here:
	--		1) the first event is SYNSS with a hash of 0 (fresh start to list) and
	--		   our targetList is empty, or
	--		2) the first event is SYNSS with a NON-zero hash, and our targetList's
	--		   hash matches that hash
	
	local currentTID = sortedTrans[1]
	if ( transList[currentTID].token == "SYNSS" ) then
		if ( transList[currentTID].nlHash == 0 ) then
			-- Our targetlist better not have any entries in it yet
			if ( next( targetList ) == nil ) then
				-- good deal. we can proceed
				SKG_Debug( "good case #1" )
			else
				SKG_Debug( "RunTransactions called, SYNSS with 0 hash" )
				SKG_Debug( "but targetlist is not empty" )
				return 1
			end
		--[[ Removing this alternate check for now. See jira SKG-113
		else
			-- we're non-zero. Match the current hash
			local thisHash = SKG_GenericListHash( currentTID, targetList )
			if ( thisHash == transList[currentTID].nlHash ) then
				-- good to go.
				SKG_Debug( "good case #2" )
			else
				SKG_Debug( "RunTransactions called, SYNSS with !0 hash" )
				SKG_Debug( "hash or targetlist != SYNSS hash" )
				return 1
			end ]]--
		end
	else
		-- we've called this routine with bad data. Try again
		SKG_Debug( "RunTransactions called, but first transaction is NOT SYNSS")
		return 1
	end

	-- loop through remaining transaction
	for transIndex = 2, #sortedTrans do
		SKG_Debug( "Transloop " .. transIndex )
		SKG_Debug( "size of targetList is " .. #targetList )
		local currentTID = sortedTrans[transIndex]
		local currentHash = transList[currentTID].nlHash
		local currentToken = transList[currentTID].token
		local currentPayload = transList[currentTID].payload

		local currentLM = strsub( currentTID, 1, 14 )
		if ( currentToken == "PLRIN" ) then
			-- payload will be { PID } or { PID, Position }
			SKG_Debug( "PLRIN" )
			local PID, Position = "", ""
			if ( #currentPayload == 2 ) then
				PID, Position = unpack( currentPayload, 1, 2 )
			else
				PID = currentPayload[1]
			end

			tinsert( targetList, PID )
			lastmodList[PID] = currentLM
			if ( Position ~= "" ) then
				for i = #targetList, ( Position + 1 ), -1 do
					targetList[i] = targetList[i-1]
				end
				targetList[Position] = PID
			end
		elseif ( currentToken == "PLRRM" ) then
			-- payload will be PID 
			SKG_Debug( "PLRRM" )

			local PID = currentPayload
			local index = 1
			local notFound = true
			while ( ( index <= #targetList ) and ( notFound == true ) ) do
				if ( targetList[index] == PID ) then
					tremove( targetList, index )
					lastmodList[PID] = nil
					notFound = false
				end
				index = index + 1
			end
		elseif ( currentToken == "PLRMV" ) then
			-- payload will be { PID, Direction }
			SKG_Debug( "PLRMV" )
			local PID, Direction = unpack( currentPayload, 1, 2 )

			-- figure out "Position"
			local Position = 0
			for i = 1, #targetList do
				if ( targetList[i] == PID ) then
					Position = i
				end
			end

			if ( Position == 0 ) then
				-- This should never happen, but check anyway
			else
				if ( Direction == "UP") then
					targetList[Position], targetList[Position - 1] =
						targetList[Position - 1], targetList[Position]
				else
					targetList[Position], targetList[Position + 1] =
						targetList[Position + 1], targetList[Position]
				end
			end
		elseif ( currentToken == "PLRSC" ) then
			-- payload is { Player, PIDList }
			SKG_Debug( "PLRSC" )

			local Player, PIDList = unpack( currentPayload, 1, 2 )
			local moveList = {}
			local lIndex = 1
			local foundFirst = false
			for i = 1, #targetList do
				local j = 1
				local notfound = true
				while ( ( j <= #PIDList ) and ( notfound == true ) ) do
					if ( targetList[i] == PIDList[j] ) then
						notfound = false
					else
						j = j + 1
					end
				end
				if ( notfound == false ) then
					if ( targetList[i] == Player ) then
						foundFirst = true
					end
					if ( foundFirst == true ) then
						moveList[lIndex] = i
						lIndex = lIndex + 1
					end
				end
			end

			local moveLen = #moveList
			local SuicideTemp = targetList[moveList[1]]
			local moveIndex = 1
			while ( moveIndex < moveLen ) do
				local movingPID = targetList[moveList[moveIndex + 1]]
				targetList[moveList[moveIndex]] = movingPID
				moveIndex = moveIndex + 1
				lastmodList[movingPID] = currentLM
			end
			targetList[moveList[moveIndex]] = SuicideTemp
			lastmodList[Player] = currentLM
		else
			-- Don't have code to deal with this token. Report:
			SKG_Debug( "In RunTransactions, got bad token: " .. currentToken )
		end
	end
	return 0
end

function SKG_tblCopy( inT ) 
	local outT = {}
	for k, v in pairs( inT ) do
		if ( type( v ) == "table" ) then
			outT[k] = SKG_tblCopy( v )
		else
			outT[k] = v
		end
	end
	return outT
end

function SKG_ImportPhase1()
	-- Note: there was no death knight in the original version. so don't worry about it here
 	SKG_Print( SKGL["SKG_MSG_IMPORT_STARTPHASE1"] )
	SKG.Import = {}
	SKG.Import.Players = {}
	SKG.Import.RollLists = {}

	-- Pass 1 : class lists
	for listName, rList in pairs( SavedRollLists ) do
		-- if its a class list, start inserting people into
		-- the Player's List
		if ( ( listName == "Warrior" ) or ( listName == "Hunter" ) or
			 ( listName == "Shaman" ) or ( listName == "Mage" ) or
			 ( listName == "Priest" ) or ( listName == "Warlock" ) or
			 ( listName == "Rogue" ) or ( listName == "Paladin" ) or
			 ( listName == "Druid" ) ) then
			for i = 1, #rList do
				local pClass = strupper( listName )
				local pName = rList[i].player
				local PID = SKG_HashName( pName )
				local t = { 
					name = pName,
					class = pClass
				}
				SKG.Import.Players[PID] = t
			end
		else
			-- only process class lists for now. We'll process Roll Lists on the seocnd pass
		end
	end

	-- I'm going to temporarily store guild info into a struct so I can compare
	-- the toons we're importing against the guild (the most common place those
	-- toons we're not sure about will be )
	local GuildRoster = {}
	local numGuildMembers = GetNumGuildMembers(true)
	for index = 1, numGuildMembers do
		local toonName, b, b, b, toonClass = GetGuildRosterInfo(index);
		local PID = SKG_HashName( toonName )
		GuildRoster[PID] = strupper( toonClass )
	end

	-- Pass 2 : roll lists
	for listName, rList in pairs( SavedRollLists ) do
		-- if its a class list, start inserting people into
		-- the Player's List
		if ( ( listName == "Warrior" ) or ( listName == "Hunter" ) or
			 ( listName == "Shaman" ) or ( listName == "Mage" ) or
			 ( listName == "Priest" ) or ( listName == "Warlock" ) or
			 ( listName == "Rogue" ) or ( listName == "Paladin" ) or
			 ( listName == "Druid" ) ) then
			-- do nothing, we handled that in pass 1
		else
			-- its a roll list
			SKG.Import.RollLists[listName] = {}
			for i = 1, #rList do
				local pName = rList[i].player
				local PID = SKG_HashName( pName ) 
				-- make the entry in the Roll List
				SKG.Import.RollLists[listName][i] = PID
			
				-- Check to see if the record exists in our Player's list
				if ( SKG.Import.Players[PID] ) then
					-- we're gtg.
				else
					-- need to make a stub entry. I'll get filled in during Phase 2
					-- check if the class is defined in the guild...
					local t = {}
					if ( GuildRoster[PID] ) then
						t = {
							name = pName,
							class = GuildRoster[PID]
						}
					else
						t = { 
							name = pName,
							class = ""
						}
					end
					SKG.Import.Players[PID] = t
				end
			end
		end
	end
	GuildRoster = {}
	SKG_Print( SKGL["SKG_MSG_IMPORT_ENDPHASE1"] )
	SKG_ImportTimerFrame:Show()
end

function SKG_CleanUpImport( ListName )
	SKG_Debug( "ForceLastMod " .. ListName )
	local db = SKG.db.profile
	local rList = db.RollLists[ListName]
	
	local newTID = SKG_CreateTID( 0, 0, 0 )
	local timeStamp = strsub( newTID, 1, 14 )
	-- if it doesn't exist, make it
	db.LastMod[ListName] = {}
	for i = 1, #rList do
		PID = rList[i]
		db.LastMod[ListName][PID] = timeStamp
	end		

	db.Snapshot[ListName] = {}
	db.Snapshot[ListName] = rList
	db.Log[ListName] = {}
	db.SnapshotLM[ListName] = {}
	db.SnapshotLM[ListName] = db.LastMod[ListName]

	local LH = SKG_HashRollList( newTID, ListName )
	SKG_AddLogEntry( newTID, ListName, "SYNSS", LH, LH )
end

function SKG_SmartCompareItemlinks( item1, item2 )
	-- My tests were able to show that successive looting attempts on the
	-- same item from the same corpse can actually produce DIFFERENT itemStrings.
	-- In all cases, the only field that changes was the uniqueid field, and in
	-- every loot attempt AFTER the first loot attempt, the unique had high-order
	-- bytes changes
	--		One example:
	--			first loot			: 37250 ( 0x00009182 )
	--			successive loots	: 1080529282 ( 0x40679182 )
	--
	-- So in order to do a real comparison, I have to do a more thorough comparison
	-- that also includes comparing just the low-order bytes of the uniqueid.
	-- 
	-- What a pain in the ass.
	--
	-- This function will return true if the itemlinks are the same, false otherwise.
	--

	-- First, try the easy way out
	if ( item1 == item2 ) then
		return true
	else
		-- redoing using patterns. Xinhuan, you bring hell to my doorstep.
		--	|cff9d9d9d|Hitem:7073:0:0:0:0:0:0:0:1|h[Broken Fang]|h|r
		local item1part1, item1UID, item1part2 = string.match( item1, 
			"|%w+|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:)(%-?%d+):%d+|h%[(.-)%]|h|r" )
		local item2part1, item2UID, item2part2 = string.match( item2,
			"|%w+|Hitem:(%d+:%d+:%d+:%d+:%d+:%d+:%-?%d+:)(%-?%d+):%d+|h%[(.-)%]|h|r" )
		
		if ( ( item1part1 == item2part1 ) and ( item1part2 == item2part2 ) ) then
			item1UID = item1UID % 65536
			item2UID = item2UID % 65536

			if ( item1UID == item2UID ) then
				return true
			end
		end
	end
	return false
end

function SKG_OpenToBidFrame()
	-- Hide all frames
	SKG_Frame:Hide()
	SKG_OptionsFrame:Hide()
	SKG_HelpFrame:Hide()
	SKG_BidFrame:Hide()
	SKG_BidButtonFrame:Hide()
	SKG_ListFrame:Hide()
	SKG_SyncFrame:Hide()
	SKG_SecurityFrame:Hide()
	-- Set the state correctly
	SKG_FRAME_STATE = "Bid"
	-- Show the right frames
	SKG_BidFrame:Show()
	SKG_Frame:Show()
	-- LOOTI handler will take care of the side frame as needed
end

function SKG_ListError( listName, errorCode )
	-- returns a string containing appropriate error text
	-- Return Codes:
	--		0	Players, Alts and requested list are all valid with no errors found
	--		11	Players list has (at least one) entry missing name or class information
	--		12	Players list has (at least one) entry with a bad hash
	--		21	Alts list has (at least one) entry whose hash is not present in the Players list
	--		22	Alts list has (at least one) entry without a valid name and class
	--		31	listName has (at least one) entry without a valid Player's list entry
	--		32	listName has (at least one) "nil" entry
	--		33	listName has (at least one) "0" entry
	--		34	listName has (at least one) skipped index
	--		40	invalid listName
	local errormsg = ""

	if ( errorCode == 0 ) then
		errormsg = "Normal"
	elseif ( errorCode == 11 ) then
		errormsg = "Players List contains entry missing name/class information"
	elseif ( errorCode == 12 ) then
		errormsg = "Players List contains entry with hash mismatch"
	elseif ( errorCode == 21 ) then
		errormsg = "Alts List contains hash not in Players list"
	elseif ( errorCode == 22 ) then
		errormsg = "Alts List contains entry missing name/class information"
	elseif ( errorCode == 31 ) then
		errormsg = "List contains entry not in Players list"
	elseif ( errorCode == 32 ) then
		errormsg = "List contains a NIL entry"
	elseif ( errorCode == 33 ) then
		errormsg = "List contains a ZERO entry"
	elseif ( errorCode == 34 ) then
		errormsg = "List contains a skipped index"
	elseif ( errorCode == 40 ) then
		errormsg = "Invalid listname" 
	end
	return "... " .. listName .. ": " .. errormsg
end

function SKG_checkList( listName )
	-- This function is called (at the moment) from a command line
	-- allowing the user to check a list for corruption.
	--
	-- Checks:
	--		Players List
	--			* verify each entry has a name and class
	--			* verify each entry's hash matches the name
	--		Alts List
	--			* verify that the hash has an entry in the Player's list
	--			* each entry has a name and class
	--		Roll Lists:
	--			* all entries have a valid Player's list entry
	--			* no "nil" entries
	--			* no '0' entries
	--			* no skipped indices
	--
	local returnCode = 0
	local pList = SKG.db.profile.Players
	local aList = SKG.db.profile.Alts
	if ( listName == "Players" ) then
		SKG_Debug( "Processing Players List" )
		for k, v in pairs( pList ) do
			if ( v.name ) then
				if ( SKG_HashName( v.name ) ~= k ) then
					returnCode = 12
					SKG_Debug( "... " .. k .. " ... Hash to Name Mismatch" )
				end
			else
				returnCode = 11
				SKG_Debug( "... " .. k .. " ... Name invalid" )
			end
			if ( v.class ) then
				-- nothing to check...
			else
				returnCode = 11
				SKG_Debug( "... " .. k .. " ... Class invalid" )
			end
		end
	elseif ( listName == "Alts" ) then
		SKG_Debug( "Processing Alts List" )
		for k, v in pairs( aList ) do
			if ( pList[k] ) then
				for k2, v2 in pairs( aList[k] ) do
					if not ( v2.name ) then
						returnCode = 22
						SKG_Debug( "... " .. k .. " ... " .. k2 .. " ... Name invalid" )
					end
					if not ( v2.class ) then
						returnCode = 22
						SKG_Debug( "... " .. k .. " ... " .. k2 .. " ... Class invalid" )
					end
				end
			else
				returnCode = 21
				SKG_Debug( "... " .. k .. " ... Alt hash not present in Players list" )
			end
		end
	else
		local rList = SKG.db.profile.RollLists[listName]
		if ( rList ) then
			SKG_Debug( "Processing list : " .. listName )
			local highestIndex = 0
			for k, v in pairs( rList ) do
				if ( pList[v] ) then
					if ( highestIndex < k ) then highestIndex = k end
					if ( v == nil ) then
						returnCode = 32
						SKG_Debug( "... I:" .. k .. " has NIL entry" )
					end
					if ( v == 0 ) then
						returnCode = 33
						SKG_Debug( "... I:" .. k .. " has ZERO entry" )
					end
				else
					-- we've got a hash in the rolllist that ISN'T in the Player's list
					SKG_Debug( "... I:" .. k .. "/H:" .. v .. " does not appear in the Player's list" )
					returnCode = 31
				end
			end
			SKG_Debug( "... Checking for missing entries..." )
			for i = 1, highestIndex do
				if not rList[i] then
					SKG_Debug( "... I:" .. i .. " is a MISSING entry." )
					returnCode = 34
				end
			end
		else
			SKG_Debug( "Invalid List" )
			returnCode = 40
		end
	end
	return returnCode
end

function SKG_RepairLists()
	-- This function will go through with a really broad stroke and repair
	-- all lists. It also will snapshot all lists as well. This is a last
	-- resort action and should be used delicately

	-- The primary method here is to simply loop through all lists (players,
	-- alts, and all roll lists), hitting every entry. Evaluate each entry to
	-- determine validity, and copy it to the target list if it meets the 
	-- validity criteria. Once completed, move the target list back to the source
	-- and snapshot.

	-- NOTE: DEBUG on for this operation will be pretty darn spammy

	SKG_Debug( "Beginning Repair Lists" )
	local db = SKG.db.profile
	-- Start with Players list
	SKG_Debug( "Repairing Players list..." )
	local targetPlayers = {}
	for playerHash, playerRecord in pairs( db.Players ) do
		-- does this entry have valid data
		if ( ( playerRecord.name ~= nil ) and
			 ( playerRecord.name ~= "" ) ) then
			local checkHash = SKG_HashName( playerRecord.name )
			if ( checkHash == playerHash ) then
				if ( ( playerRecord.class ~= nil ) and
					 ( playerRecord.class ~= "" ) ) then
					-- does the record already exist in the target list?
					if ( targetPlayers[playerHash] ) then
						-- it already exists, don't copy in and make a duplicate entry
						SKG_Debug( "...Player record " .. playerHash .. " already in target list" )
					else
						targetPlayers[playerHash] = {}
						targetPlayers[playerHash].name = playerRecord.name
						targetPlayers[playerHash].class = playerRecord.class
					end
				else
					SKG_Debug( "...Player record " .. playerHash .. " has invalid class" )
				end
			else
				SKG_Debug( "...Hash for record " .. playerHash .. " does not match name" )
			end
		else
			SKG_Debug( "...Player record " .. playerHash .. " has invalid name" )
		end
	end
	db.Players = nil
	db.Players = SKG_tblCopy( targetPlayers )

	-- Next up, Alts list
	SKG_Debug( "Repairing Alts list..." )
	local targetAlts = {}
	for mainPID, AltsList in pairs( db.Alts ) do
		-- does this mainPID exist in the players list? If not, we can ignore the rest...
		if ( db.Players[mainPID] ) then
			-- ok, proceed
			targetAlts[mainPID] = {}
			SKG_Debug( "...Looping through alts for " .. mainPID )
			for altIndex, altRecord in pairs( db.Alts[mainPID] ) do
				if ( ( altRecord.name ~= nil ) and
					 ( altRecord.name ~= "" ) ) then
					if ( ( altRecord.class ~= nil ) and
						 ( altRecord.class ~= "" ) ) then
						-- For the moment, I am *NOT* going to check to make sure that
						-- alt records aren't dupes. I've never seen this in any reported 
						-- problem. If it crops up, I can always add it in later.
						local t = {}
						t.name = altRecord.name
						t.class = altRecord.class
						tinsert( targetAlts[mainPID], t )
					else
						SKG_Debug( "... ...Alt with index " .. altIndex .. " has invalid class" )
					end
				else
					SKG_Debug( "... ...Alt with index " .. altIndex .. " has invalid name" )
				end
			end
		else
			SKG_Debug( "...Alt record " .. mainPID .. " does not exist in Players list. Ignoring" )
		end
	end
	db.Alts = nil
	db.Alts = SKG_tblCopy( targetAlts )

	-- Finally, loop through our roll lists...
	SKG_Debug( "Repairing Roll Lists..." )
	-- establish a new "lastMod" to use for all entries.
	local TID = SKG_CreateTID( 0, 0, 0 )
	local lastMod = strsub( TID, 1, 14 )
	for listName, rList in pairs( db.RollLists ) do
		local targetLMList = {}
		local targetRList = {}
		local secondRList = {}
		-- first, go through the whole list and move over valid entries. Note the highest index
		-- so we can loop again after and remove blank entries
		SKG_Debug( "...Repairing " .. listName )
		local maxIndex = 0
		for lIndex, playerHash in pairs( rList ) do
			-- make sure the entry exists in the Players list
			if ( db.Players[playerHash] ) then
				tinsert( targetRList, lIndex, playerHash )
				if ( lIndex > maxIndex ) then
					maxIndex = lIndex
				end
			else
				SKG_Debug( "... ...Entry " .. playerHash .. " not found in Players" )
			end
		end
		-- LibStub("AceConsole-2.0"):PrintLiteral( targetRList )
		-- second, go through all entries in a sorted linear loop, and remove any blank
		-- entries
		for loopy = 1, maxIndex do
			SKG_Debug( "loopy = " .. loopy ) 
			-- check for nil, 0, or empty entries. remove them
			if ( ( targetRList[loopy] ~= nil ) and
				 ( targetRList[loopy] ~= 0 ) and
				 ( targetRList[loopy] ~= "" ) ) then
				 tinsert( secondRList, targetRList[loopy] )
			end
			-- LibStub("AceConsole-2.0"):PrintLiteral( secondRList )
		end
		-- what should be left at this point is a good list with no holes.
		-- Make out LM list
		for loopy = 1, #secondRList do
			targetLMList[ secondRList[loopy] ] = lastMod
		end
		-- push the lists back to their sources
		db.RollLists[listName] = nil
		db.LastMod[listName] = nil
		db.RollLists[listName] = SKG_tblCopy( secondRList )
		db.LastMod[listName] = SKG_tblCopy( targetLMList )
		-- Finally, force a snapshot of the list to get us back to square one.
		local latestTID = SKG_GetLatestTID( db.Log[listName] )
		local LH, SH = SKG_HashRollList( latestTID, listName )
		db.Snapshot[listName] = {}
		db.Snapshot[listName] = SKG_tblCopy ( db.RollLists[listName] )
		db.Log[listName] = {}
		db.SnapshotLM[listName] = {}
		db.SnapshotLM[listName] = SKG_tblCopy ( db.LastMod[listName] )
		SKG_AddLogEntry( TID, listName, "SYNSS", LH, LH )
	end
	-- Finally, clean stuff up
	SKG_UpdateSPlayers()
	SKG_UpdateMainsChooser()
	SKG_UpdateToonChooser()
	SKG_UpdateCurrentList()
end

function SKG_Toggle()
	if ( SKG_Frame:IsVisible() ) then
		PlaySound("igMainMenuClose");
		SKG_Frame:Hide()
	else
		PlaySound("igMainMenuOpen");
		SKG_Frame:Show()
	end
end

function SKG_ShowConfig( parentFrame )
	Dewdrop:Register( parentFrame,
		'children', fubarOptionsTable,
		'dontHook', true,
		'point', "TOPLEFT",
		'relativePoint', "BOTTOMLEFT"
	)
	this:SetScript( "OnClick",
		function()
			if Dewdrop:IsOpen( this ) then
				Dewdrop:Close()
			else
				Dewdrop:Open( this )
			end
		end
	)
	this:GetScript( "OnClick" )()
end

--[[
************************************************************************
CHANGELOG:
Date : 4/20/09
	Removed old security method/functions
	Added nil check for raidClass in UpdateRaiders
Date : 4/14/09
	Removed references to math.randomseed. Lame
Date : 12/21/08
	Added in changes for Death Knight compatibility. I think I got
	everything.
Date : 10/25/08
	Initial attempt with Ace3 instead of LibRock. Probably will require
	major edits once. Added header/footer. Made changes to itemLinks
	that were updated with a new field in 3.0.2.
************************************************************************
]]--